@@ -17,10 +17,10 @@ use configparser::ini;
 use structopt::StructOpt;
 use wildmatch::WildMatch;
 
-/// The final `process::Command` to execute, bundled together with related information for quick
-/// access.  Those related path informations must be set manually when building the `cmdline`.
-/// They point to the arguments used in the command, such as the path to a `game`.  The `output`
-/// must be set manually after executing the `cmdline` process.
+/// The final `process::Command` to execute and run `retroarch`.  It bundles related information
+/// such as paths and the `output` from stdout.  The additional path data should be manually set
+/// when building the main `cmdline`, to stay in sync.  The `output` must be set manually after
+/// executing `cmdline` process.
 #[derive(Debug)]
 pub struct RunCommand {
     pub cmdline: Command,
@@ -36,6 +36,7 @@ pub struct RunCommand {
 #[derive(Debug)]
 pub struct Settings {
     games: Vec<PathBuf>,
+    retroarch_arguments: Vec<String>,
     config: Option<PathBuf>,
     retroarch: Option<PathBuf>,
     retroarch_config: Option<PathBuf>,
@@ -66,6 +67,7 @@ impl Settings {
     pub fn new() -> Settings {
         Settings {
             games: vec![],
+            retroarch_arguments: vec![],
             config: None,
             retroarch: None,
             retroarch_config: None,
@@ -127,6 +129,7 @@ impl Settings {
         // list
         // Take it, as it is always a list.
         settings.games = args.games;
+        settings.retroarch_arguments = args.retroarch_arguments;
 
         // Option
         // Take them, as they are optional anyway.
@@ -297,13 +300,13 @@ impl Settings {
     /// the same options found in the programs commandlline arguments (use `enjoy --help` to find
     /// out more).  There are only a couple of differences to it:
     ///
-    ///     - There is only one `game` entry instead of a list.
-    ///     - Some options about the configuration file itself are not present here, because they
-    ///     are evaluated before loading the INI: `--config`, `--open-config` or `--noconfig`.
-    ///     - Flags in the commandline can be used here with a value of `1` or `true` to indicate
-    ///     they are active.  This is needed, because each key or option in the INI file has a
-    ///     value, which is not the case on the commandline.  An example would be `--norun`, which
-    ///     is then translated into INI key `norun = 1`.
+    /// 1. There is only one `game` entry instead of a list.
+    /// 2. Some options about the configuration file itself are not present here, because they are
+    ///    evaluated before loading the INI: `--config`, `--open-config` or `--noconfig`.
+    /// 3. Flags in the commandline can be used here with a value of `1` or `true` to indicate they
+    ///    are active.  This is needed, because each key or option in the INI file has a value,
+    ///    which is not the case on the commandline.  An example would be `--norun`, which is then
+    ///    translated into INI key `norun = 1`.
     ///
     /// ```ini
     /// [options]
@@ -318,6 +321,10 @@ impl Settings {
             if let Some(value) = ini.get("options", "game") {
                 settings.games.push(PathBuf::from(value));
             }
+            if let Some(value) = ini.get("options", "retroarch_arguments") {
+                settings.retroarch_arguments =
+                    shlex::split(&value).unwrap_or_default();
+            }
             if let Some(value) = ini.get("options", "retroarch") {
                 settings.retroarch = Some(PathBuf::from(value));
             }
@@ -500,6 +507,15 @@ impl Settings {
             }
         }
 
+        if !overwrite.retroarch_arguments.is_empty() {
+            if self.retroarch_arguments.is_empty() {
+                self.retroarch_arguments = overwrite.retroarch_arguments;
+            } else {
+                self.retroarch_arguments
+                    .append(&mut overwrite.retroarch_arguments.clone());
+            }
+        }
+
         if overwrite.config.is_some() {
             self.config = overwrite.config;
         }
@@ -668,6 +684,11 @@ impl Settings {
             command.arg("--fullscreen");
         }
 
+        // `--`
+        if !self.retroarch_arguments.is_empty() {
+            command.args(self.retroarch_arguments.iter());
+        }
+
         // Use `run.cmdline` to get the full command with all options to be executed.  `output`
         // needs to be updated manually, by catching the output when running the `cmdline`.
         let run = RunCommand {
@@ -862,6 +883,7 @@ mod tests {
     fn new_from_defaults_retroarch() -> Result<(), Box<dyn Error>> {
         let settings = super::Settings {
             games: vec![],
+            retroarch_arguments: vec![],
             config: None,
             retroarch: Some(PathBuf::from("retroarch")),
             retroarch_config: None,
@@ -943,6 +965,7 @@ mod tests {
             "
             [options]
             retroarch = /usr/bin/retroarch
+            retroarch_arguments = --verbose --set-shader \"\"
             which = 0
             norun = true
             libretro_directory=
@@ -997,6 +1020,26 @@ mod tests {
         cores_rules
     }
 
+    #[test]
+    fn read_config_options_retroarch_arguments() -> Result<(), Box<dyn Error>>
+    {
+        let mut settings = super::Settings::new();
+        let ini = test_ini_template();
+
+        super::Settings::read_config_options(
+            &mut settings,
+            &ini,
+            &["options".to_string()],
+        )?;
+
+        assert_eq!(
+            vec!["--verbose", "--set-shader", ""],
+            settings.retroarch_arguments
+        );
+
+        Ok(())
+    }
+
     #[test]
     fn read_config_options_path() -> Result<(), Box<dyn Error>> {
         let mut settings = super::Settings::new();
@@ -1092,6 +1135,7 @@ mod tests {
         let mut old = super::Settings::new();
         let new = super::Settings {
             games: vec![],
+            retroarch_arguments: vec![],
             config: None,
             retroarch: Some(PathBuf::from("retroarch")),
             retroarch_config: None,
@@ -1132,6 +1176,7 @@ mod tests {
                 .collect();
         let mut settings = super::Settings {
             games: games,
+            retroarch_arguments: vec![],
             config: None,
             retroarch: Some(PathBuf::from("retroarch")),
             retroarch_config: None,
